use std::io;
use std::thread;
use std::pin::Pin;
use std::task::{Context, Poll};

use linefeed::{Interface, ReadResult};
use linefeed::memory::MemoryTerminal;

use futures::Stream;
use futures::channel::mpsc;
use futures::SinkExt;

pub struct Inputs {
    rx: mpsc::Receiver<String>,
    hndl: thread::JoinHandle<()>,
}

impl Inputs {
    pub fn new(term: MemoryTerminal) -> Self {
        let (mut tx, rx) = mpsc::channel(64);

        let hndl = thread::spawn(move || {
            let stdin = io::stdin();
            let mut reader = Interface::with_term("sute", term).unwrap();
            reader.set_prompt("> ").unwrap();
            while let ReadResult::Input(line) = reader.read_line().unwrap() {
                smol::block_on(tx.send(line));
            }
        });

        Self { rx, hndl }
    }
}

impl Stream for Inputs {
    type Item = String;

    fn poll_next(mut self: Pin<&mut Self>, cx: &mut Context) -> Poll<Option<Self::Item>> {
        Pin::new(&mut self.rx).poll_next(cx)
    }
}
